@@ -17,24 +17,55 @@ from distutils.version import LooseVersion
 
 import horovod.tensorflow as hvd
 import tensorflow as tf
+from horovod.tensorflow.gradient_aggregation import LocalGradientAggregationHelper
+from horovod.tensorflow.gradient_aggregation_eager import LocalGradientAggregationHelperEager
+from horovod.tensorflow.mpi_ops import rank
 
 
 _PRE_TF_2_4_0 = LooseVersion(tf.__version__) < LooseVersion('2.4.0')
 
 
 def create_distributed_optimizer(keras, optimizer, name, device_dense, device_sparse,
-                                 compression, sparse_as_dense, gradient_predivide_factor):
+                                 compression, sparse_as_dense, gradient_predivide_factor,
+                                 op, backward_passes_per_step=1,
+                                 average_aggregated_gradients=False,
+                                 num_groups=0):
     class _DistributedOptimizer(keras.optimizers.Optimizer):
         _HAS_AGGREGATE_GRAD = True
 
         def __init__(self, **kwargs):
             self._name = name or "Distributed%s" % self.__class__.__base__.__name__
-            self._device_dense = device_dense
-            self._device_sparse = device_sparse
-            self._compression = compression
-            self._sparse_as_dense = sparse_as_dense
             self._aggregated_gradients = False
-            self._gradient_predivide_factor = gradient_predivide_factor
+
+            self._allreduce_grads = hvd._make_allreduce_grads_fn(
+                self._name,
+                device_dense,
+                device_sparse,
+                compression,
+                sparse_as_dense,
+                op,
+                gradient_predivide_factor,
+                num_groups)
+
+            self._agg_helper = None
+            if backward_passes_per_step > 1:
+                if hvd._executing_eagerly():
+                    self._agg_helper = LocalGradientAggregationHelperEager(
+                        backward_passes_per_step=backward_passes_per_step,
+                        allreduce_func=self._allreduce_grads,
+                        sparse_as_dense=sparse_as_dense,
+                        average_aggregated_gradients=average_aggregated_gradients,
+                    )
+                else:
+                    self._agg_helper = LocalGradientAggregationHelper(
+                        backward_passes_per_step=backward_passes_per_step,
+                        allreduce_func=self._allreduce_grads,
+                        sparse_as_dense=sparse_as_dense,
+                        average_aggregated_gradients=average_aggregated_gradients,
+                        rank=rank(),
+                        optimizer_type=LocalGradientAggregationHelper._OPTIMIZER_TYPE_KERAS,
+                    )
+
             super(self.__class__, self).__init__(**kwargs)
 
         def get_gradients(self, loss, params):
@@ -58,48 +89,40 @@ def create_distributed_optimizer(keras, optimizer, name, device_dense, device_sp
                 return aggregated_grads
             return list(zip(aggregated_grads, vars))
 
-        def _allreduce(self, gradients):
+        def _allreduce(self, grads):
             self._aggregated_gradients = True
-            if hvd.size() > 1:
-                if self._gradient_predivide_factor != 1.0:
-                    # Perform averaging via pre/postscaling factors.
-                    # Split average operation across pre/postscale factors
-                    prescale_factor = 1.0 / gradient_predivide_factor
-                    postscale_factor = gradient_predivide_factor / hvd.size()
-                    do_average = False
-                else:
-                    prescale_factor = 1.0
-                    postscale_factor = 1.0
-                    do_average = True
 
-                averaged_gradients = []
-                with tf.name_scope(self._name + "_Allreduce"):
-                    for grad in gradients:
-                        if grad is not None:
-                            if self._sparse_as_dense and \
-                                    isinstance(grad, tf.IndexedSlices):
-                                grad = tf.convert_to_tensor(grad)
-                            avg_grad = hvd.allreduce(grad,
-                                                     average=do_average,
-                                                     device_dense=self._device_dense,
-                                                     device_sparse=self._device_sparse,
-                                                     compression=self._compression,
-                                                     prescale_factor=prescale_factor,
-                                                     postscale_factor=postscale_factor)
-                            averaged_gradients.append(avg_grad)
-                        else:
-                            averaged_gradients.append(None)
-                    return averaged_gradients
+            if self._agg_helper:
+                return self._agg_helper.compute_gradients(tuple(grads))
             else:
-                return gradients
+                return self._allreduce_grads(grads)
 
         def apply_gradients(self, *args, **kwargs):
-            results = super(self.__class__, self).apply_gradients(*args, **kwargs)
+            if self._agg_helper:
+                if isinstance(args[0], zip):
+                    # If grad_and_vars are passed in as a zip object
+                    # convert to a list. This is necessary for TF2.4+
+                    # b/c args[0] is used in both conditional branches
+                    # inside _agg_helper.apply_gradients().
+                    args = list(args)
+                    args[0] = list(args[0])
+                    args = tuple(args)
+
+                results = self._agg_helper.apply_gradients(
+                    lambda: super(self.__class__, self).apply_gradients(*args, **kwargs),
+                    self,
+                    *args,
+                    **kwargs,
+                )
+            else:
+                results = super(self.__class__, self).apply_gradients(*args, **kwargs)
+
             if not self._aggregated_gradients:
                 raise Exception('`apply_gradients()` was called without a call to '
                                 '`get_gradients()` or `_aggregate_gradients`. If you\'re '
                                 'using TensorFlow 2.0, please specify '
                                 '`experimental_run_tf_function=False` in `compile()`.')
+
             return results
 
     # We dynamically create a new class that inherits from the optimizer that was passed in.
@@ -108,6 +131,7 @@ def create_distributed_optimizer(keras, optimizer, name, device_dense, device_sp
     # model could be easily restored without Horovod.
     cls = type(optimizer.__class__.__name__, (optimizer.__class__,),
                dict(_DistributedOptimizer.__dict__))
+
     return cls.from_config(optimizer.get_config())
 
 
@@ -123,10 +147,11 @@ if hasattr(hvd, 'broadcast_global_variables'):
         return _eval(backend, hvd.broadcast_global_variables(root_rank))
 
 
-def allreduce(backend, value, name, average, prescale_factor, postscale_factor):
+def allreduce(backend, value, name, average, prescale_factor, postscale_factor, op, compression):
     return _eval(backend, hvd.allreduce(tf.constant(value, name=name), average=average,
                                         prescale_factor=prescale_factor,
-                                        postscale_factor=postscale_factor))
+                                        postscale_factor=postscale_factor,
+                                        op=op, compression=compression))
 
 
 def allgather(backend, value, name):
