@@ -1,14 +1,19 @@
 import ray
+from ray import services
 
 from collections import defaultdict
 from dataclasses import dataclass
 import os
+import socket
 from typing import Dict, Callable, Any, Optional, List
 import logging
 
 from horovod.runner.common.util import secret, timeout, hosts
 from horovod.runner.driver import driver_service
 from horovod.runner.http.http_server import RendezvousServer
+from horovod.runner.util import network
+
+from horovod.ray.driver_service import _driver_fn
 
 logger = logging.getLogger(__name__)
 
@@ -44,13 +49,14 @@ class BaseHorovodWorker:
     executable = None
 
     def __init__(self, world_rank=0, world_size=1):
+        os.environ["HOROVOD_HOSTNAME"] = self.hostname()
         os.environ["HOROVOD_RANK"] = str(world_rank)
         os.environ["HOROVOD_SIZE"] = str(world_size)
 
     def hostname(self) -> str:
         # TODO: This is probably not the right way to retrieve
         # the intended hostname.
-        return ray.services.get_node_ip_address()
+        return socket.gethostname()
 
     def update_env_vars(self, env_vars: Dict[str, str]):
         """Update the env vars in the actor process."""
@@ -132,17 +138,16 @@ class NodeColocator:
         # Create a node ip resource label so that we can pin
         # all of the child actors to the same node. This ensures
         # colocation and balanced training.
-        node_id = f"node:{ray.services.get_node_ip_address()}"
+        node_id = f"node:{services.get_node_ip_address()}"
         remote_cls = ray.remote(BaseHorovodWorker)
         remote_cls = remote_cls.options(
             num_cpus=0, num_gpus=0, resources={node_id: 0.01})
 
-        world_rank_start = self.num_slots * self.node_rank
+        rank_start = self.num_slots * self.node_rank
 
         self.workers = [
             remote_cls.remote(world_rank=rank, world_size=self.world_size)
-            for rank in range(world_rank_start, world_rank_start +
-                              self.num_slots)
+            for rank in range(rank_start, rank_start + self.num_slots)
         ]
 
         # Propogate cuda visible devices to the underlying
@@ -165,6 +170,10 @@ class NodeColocator:
     def get_workers(self) -> List:
         return self.workers
 
+    def execute(self, func):
+        """Executes an arbitrary function on self."""
+        return func(self)
+
 
 class Coordinator:
     """Responsible for instantiating the Rendezvous server.
@@ -229,17 +238,12 @@ class Coordinator:
         # start global rendezvous server and get port that it is listening on
         self.global_rendezv_port = self.rendezvous.start()
         self.rendezvous.init(host_alloc_plan)
-        # remote_host_names = network.filter_local_addresses()
-        self.nics = driver_service.get_common_interfaces(
-            self.settings, list(self.hostnames_by_rank))
 
         return {
-            "HOROVOD_GLOO_RENDEZVOUS_ADDR": ray.services.get_node_ip_address(),
+            "HOROVOD_GLOO_RENDEZVOUS_ADDR": services.get_node_ip_address(),
             "HOROVOD_GLOO_RENDEZVOUS_PORT": str(self.global_rendezv_port),
             "HOROVOD_CONTROLLER": "gloo",
             "HOROVOD_CPU_OPERATIONS": "gloo",
-            "HOROVOD_GLOO_IFACE": str(list(self.nics)[0]),  # TODO
-            "NCCL_SOCKET_IFNAME": ",".join(self.nics),  # TDOO
         }
 
 
@@ -252,9 +256,11 @@ class RayExecutor:
             from RayExecutor.create_settings.
         num_hosts (int): Number of machines to execute the job on.
         num_slots (int): Humber of workers to be placed on each machine.
+        cpus_per_slot (int): Number of CPU resources to allocate to
+            each worker.
         use_gpu (bool): Whether to use GPU for allocation. TODO: this
             can be removed.
-        cpus_per_slot (int): Number of CPU resources to allocate to
+        gpus_per_slot (int): Number of GPU resources to allocate to
             each worker.
     """
 
@@ -284,12 +290,22 @@ class RayExecutor:
                  num_hosts: int = 1,
                  num_slots: int = 1,
                  cpus_per_slot: int = 1,
-                 use_gpu: bool = False):
+                 use_gpu: bool = False,
+                 gpus_per_slot: Optional[int] = None):
+
+        if gpus_per_slot and not use_gpu:
+            raise ValueError("gpus_per_slot is set, but use_gpu is False. "
+                             "use_gpu must be True if gpus_per_slot is set. ")
+        if use_gpu and isinstance(gpus_per_slot, int) and gpus_per_slot < 1:
+            raise ValueError(
+                f"gpus_per_slot must be >= 1: Got {gpus_per_slot}.")
+
         self.settings = settings
         self.num_hosts = num_hosts
         self.num_slots = num_slots
         self.cpus_per_slot = cpus_per_slot
         self.use_gpu = use_gpu
+        self.gpus_per_slot = gpus_per_slot or 1
 
     @property
     def num_workers(self):
@@ -325,6 +341,32 @@ class RayExecutor:
         workers = map_blocking(lambda w: w.get_workers.remote(), colocators)
         return sum(workers, [])
 
+    def detect_nics(self):
+        """Decomposed version of driver_service.get_common_interfaces()."""
+        nics = None
+        all_host_names = list(self.coordinator.hostnames_by_rank)
+        remote_host_names = network.filter_local_addresses(all_host_names)
+        if len(remote_host_names) > 0:
+            nics = self.settings.nics
+            if not nics:
+                if self.settings.verbose >= 2:
+                    print('Testing interfaces on all hosts.')
+
+                local_host_names = set(all_host_names) - set(remote_host_names)
+                nics = _driver_fn(self.colocators, all_host_names,
+                                  local_host_names, self.settings)
+
+                if self.settings.verbose >= 2:
+                    print('Interfaces on all hosts were successfully checked.')
+                    print('Common interface found: ' + ' '.join(nics))
+        else:
+            nics = driver_service.get_local_interfaces(self.settings)
+
+        return {
+            "HOROVOD_GLOO_IFACE": list(nics)[0],
+            "NCCL_SOCKET_IFNAME": ",".join(nics),  # TODO
+        }
+
     def start(self,
               executable_cls: type = None,
               executable_args: Optional[List] = None,
@@ -353,7 +395,7 @@ class RayExecutor:
 
         def resources_per_host():
             num_cpus = self.cpus_per_slot * self.num_slots
-            num_gpus = self.num_slots * int(self.use_gpu)
+            num_gpus = self.gpus_per_slot * self.num_slots * int(self.use_gpu)
             return dict(num_cpus=num_cpus, num_gpus=num_gpus)
 
         self.coordinator = Coordinator(self.settings)
@@ -363,17 +405,6 @@ class RayExecutor:
             executable_cls=executable_cls,
             executable_args=executable_args,
             executable_kwargs=executable_kwargs)
-
-        # Update the environment variables.
-        local_hostname = str(ray.services.get_node_ip_address())
-        ray.get([
-            w.update_env_vars.remote(
-                dict(
-                    HOROVOD_HOSTNAME=local_hostname,
-                    HOROVOD_RANK=i,
-                    HOROVOD_SIZE=self.num_workers))
-            for i, w in enumerate(self.workers)
-        ])
         # Get all the hostnames of all workers
         hostnames = map_blocking(lambda w: w.hostname.remote(), self.workers)
         # Register each hostname to the coordinator. assumes the hostname
@@ -388,6 +419,7 @@ class RayExecutor:
 
         coordinator_envs = self.coordinator.establish_rendezvous()
         coordinator_envs.update(extra_env_vars)
+        coordinator_envs.update(self.detect_nics())
 
         map_blocking(lambda w: w.update_env_vars.remote(coordinator_envs),
                      self.workers)
@@ -421,9 +453,10 @@ class RayExecutor:
         """
         args = args or []
         kwargs = kwargs or {}
-        return ray.get(
-            [worker.execute.remote(
-                lambda w: fn(*args, **kwargs)) for worker in self.workers])
+        return ray.get([
+            worker.execute.remote(lambda w: fn(*args, **kwargs))
+            for worker in self.workers
+        ])
 
     def execute_single(self,
                        fn: Callable[["executable_cls"], Any]) -> List[Any]:
