@@ -20,6 +20,7 @@ import io
 import os
 import sys
 import textwrap
+import warnings
 
 import yaml
 
@@ -50,6 +51,8 @@ CACHE_STALENESS_THRESHOLD_MINUTES = 60
 # Number of attempts for sshing into the hosts
 SSH_ATTEMPTS = 5
 
+SSH_CONNECT_TIMEOUT_S = 10
+
 
 @cache.use_cache()
 def _check_all_hosts_ssh_successful(host_addresses, ssh_port=None, ssh_identity_file=None):
@@ -83,7 +86,8 @@ def _check_all_hosts_ssh_successful(host_addresses, ssh_port=None, ssh_identity_
     args_list = [[get_remote_command(local_command='true',
                                      host=host_address,
                                      port=ssh_port,
-                                     identity_file=ssh_identity_file)]
+                                     identity_file=ssh_identity_file,
+                                     timeout_s=SSH_CONNECT_TIMEOUT_S)]
                  for host_address in host_addresses]
     ssh_exit_codes = \
         threads.execute_function_multithreaded(exec_command,
@@ -209,6 +213,32 @@ def make_override_false_action(override_args):
     return make_override_bool_action(override_args, False)
 
 
+def make_deprecated_bool_action(override_args, bool_value, replacement_option):
+    class StoreOverrideBoolAction(argparse.Action):
+        def __init__(self,
+                     option_strings,
+                     dest,
+                     required=False,
+                     help=None):
+            super(StoreOverrideBoolAction, self).__init__(
+                option_strings=option_strings,
+                dest=dest,
+                const=bool_value,
+                nargs=0,
+                default=None,
+                required=required,
+                help=help)
+
+        def __call__(self, parser, args, values, option_string=None):
+            deprecated_option = '|'.join(self.option_strings)
+            warnings.warn(f'Argument {deprecated_option} has been replaced by {replacement_option} and will be removed in v0.21.0',
+                          DeprecationWarning)
+            override_args.add(self.dest)
+            setattr(args, self.dest, self.const)
+
+    return StoreOverrideBoolAction
+
+
 def parse_args():
     override_args = set()
 
@@ -276,12 +306,12 @@ def parse_args():
                               help='Fusion buffer threshold in MB. This is the maximum amount of '
                                    'tensor data that can be fused together into a single batch '
                                    'during allreduce / allgather. Setting 0 disables tensor fusion. '
-                                   '(default: 64)')
+                                   '(default: 128)')
     group_params.add_argument('--cycle-time-ms', action=make_override_action(override_args), type=float,
                               help='Cycle time in ms. This is the delay between each tensor fusion '
                                    'cycle. The larger the cycle time, the more batching, but the '
                                    'greater latency between each allreduce / allgather operations. '
-                                   '(default: 5')
+                                   '(default: 1')
     group_params.add_argument('--cache-capacity', action=make_override_action(override_args), type=int,
                               help='Maximum number of tensor names that will be cached to reduce amount '
                                    'of coordination required between workers before performing allreduce / '
@@ -426,10 +456,23 @@ def parse_args():
                                choices=config_parser.LOG_LEVELS,
                                help='Minimum level to log to stderr from the Horovod backend. (default: WARNING).')
     group_logging_timestamp = group_logging.add_mutually_exclusive_group()
-    group_logging_timestamp.add_argument('--log-hide-timestamp', action=make_override_true_action(override_args),
-                                         help='Hide the timestamp from Horovod log messages.')
-    group_logging_timestamp.add_argument('--no-log-hide-timestamp', dest='log_hide_timestamp',
-                                         action=make_override_false_action(override_args), help=argparse.SUPPRESS)
+    group_logging_timestamp.add_argument('--log-with-timestamp', 
+                                         action=make_override_true_action(override_args),
+                                         help=argparse.SUPPRESS)
+    group_logging_timestamp.add_argument('--log-without-timestamp', dest='log_with_timestamp',
+                                         action=make_override_false_action(override_args), 
+                                         help='Hide the timestamp from Horovod internal log messages.')
+    group_logging_timestamp.add_argument('-prefix-timestamp', '--prefix-output-with-timestamp', action='store_true',
+                                         dest='prefix_output_with_timestamp',
+                                         help='Timestamp each line of output to stdout, stderr, and stddiag.')
+    group_logging_timestamp.add_argument('--log-hide-timestamp', 
+                                         dest='log_with_timestamp',
+                                         action=make_deprecated_bool_action(override_args, False, '--log-without-timestamp'),
+                                         help=argparse.SUPPRESS)
+    group_logging_timestamp.add_argument('--no-log-hide-timestamp', 
+                                         dest='log_with_timestamp',
+                                         action=make_deprecated_bool_action(override_args, True, '--log-with-timestamp'),
+                                         help=argparse.SUPPRESS)                                     
 
     group_hosts_parent = parser.add_argument_group('host arguments')
     group_hosts = group_hosts_parent.add_mutually_exclusive_group()
@@ -482,8 +525,6 @@ def parse_args():
 
 
 def _run_static(args):
-    nics_set = set(args.nics.split(',')) if args.nics else None
-
     # horovodrun has to finish all the checks before this timeout runs out.
     if args.start_timeout:
         start_timeout = args.start_timeout
@@ -508,7 +549,8 @@ def _run_static(args):
                                      hosts=args.hosts,
                                      output_filename=args.output_filename,
                                      run_func_mode=args.run_func is not None,
-                                     nics=nics_set)
+                                     nics=args.nics,
+                                     prefix_output_with_timestamp=args.prefix_output_with_timestamp)
 
     # This cache stores the results of checks performed by horovod
     # during the initialization step. It can be disabled by setting
@@ -612,7 +654,8 @@ def _run_elastic(args):
                                                 start_timeout=tmout,
                                                 output_filename=args.output_filename,
                                                 run_func_mode=args.run_func is not None,
-                                                nics=args.nics)
+                                                nics=args.nics,
+                                                prefix_output_with_timestamp=args.prefix_output_with_timestamp)
 
     if not gloo_built(verbose=(settings.verbose >= 2)):
         raise ValueError('Gloo support is required to use elastic training, but has not been built.  Ensure CMake is '
@@ -706,6 +749,9 @@ def _run(args):
             # Set hosts to localhost if not specified
             args.hosts = 'localhost:{np}'.format(np=args.np)
 
+    # Convert nics into set
+    args.nics = set(args.nics.split(',')) if args.nics else None
+
     if _is_elastic(args):
         return _run_elastic(args)
     else:
