@@ -13,16 +13,17 @@
 # limitations under the License.
 # ==============================================================================
 
+import logging
 import threading
 import traceback
 
-from horovod.run.util.threads import on_event
+from horovod.runner.util.threads import on_event
 from horovod.spark.task import task_service
 from horovod.spark.driver import driver_service
 
 
 def rsh(driver_addresses, key, host_hash, command, env, local_rank, verbose,
-        background=True, event=None):
+        background=True, events=None):
     """
     Method to run a command remotely given a host hash, local rank and driver addresses.
 
@@ -43,7 +44,8 @@ def rsh(driver_addresses, key, host_hash, command, env, local_rank, verbose,
     :param local_rank: local rank on the host of task to run the command in
     :param verbose: verbosity level
     :param background: run command in background if True, returns command result otherwise
-    :param event: event to abort the command, only if background is True
+    :param events: events to abort the command, only if background is True
+    :return exit code if background is False
     """
     if ':' in host_hash:
         raise Exception('Illegal host hash provided. Are you using Open MPI 4.0.0+?')
@@ -56,16 +58,17 @@ def rsh(driver_addresses, key, host_hash, command, env, local_rank, verbose,
     task_client.run_command(command, env)
 
     if not background:
-        stop = None
-        if event is not None:
-            stop = threading.Event()
+        events = events or []
+        stop = threading.Event()
+        for event in events:
             on_event(event, task_client.abort_command, stop=stop)
 
         try:
-            return task_client.wait_for_command_exit_code()
+            exit_code = task_client.wait_for_command_exit_code()
+            logging.debug('rsh exit code %s for host %s slot %s', exit_code, host_hash, local_rank)
+            return exit_code
         except:
             traceback.print_exc()
             return -1
         finally:
-            if stop is not None:
-                stop.set()
+            stop.set()
